<?php

/**
 * @file
 * Template overrides as well as (pre-)process and alter hooks for the
 * WCM Omega theme.
 */

// force regions to display without blocks
function wcm_omega_page_alter(&$page) {
  $regions = system_region_list($GLOBALS['theme'], REGIONS_ALL);

  if (array_key_exists('osu_navbar', $regions)) {
		$page['osu_navbar'] = array(
			'#region' => 'osu_navbar',
			'#weight' => '-10',
			'#theme_wrappers' => array('region'),
		);
	}

  if (array_key_exists('masthead', $regions)) {
		$page['masthead'] = array(
			'#region' => 'masthead',
			'#weight' => '-9',
			'#theme_wrappers' => array('region'),
		);
	}

  if (array_key_exists('footer_1', $regions)) {
		$page['footer_1'] = array(
			'#region' => 'footer_1',
			'#weight' => '-7',
			'#theme_wrappers' => array('region'),
		);
	}

  if (empty($page['footer_2']) && array_key_exists('footer_2', $regions)) {
    $page['footer_2'] = array(
      '#region' => 'footer_2',
      '#weight' => '-7',
      '#theme_wrappers' => array('region'),
    );
  }

  if (empty($page['footer_3']) && array_key_exists('footer_3', $regions)) {
    $page['footer_3'] = array(
      '#region' => 'footer_3',
      '#weight' => '-7',
      '#theme_wrappers' => array('region'),
    );
  }
}


//custom file icons
function wcm_omega_file_icon($variables) {
	$file = $variables['file'];
  $icon_directory = drupal_get_path('theme', 'wcm_omega') . '/images/file-icons';
  $mime = check_plain($file->filemime);
  $icon_url = file_icon_url($file, $icon_directory);
  return '<img alt="" class="file-icon" src="' . $icon_url . '" title="' . $mime . '" />';
}


/*
 * Implements hook_omega_theme_libraries_info().
 */
function wcm_omega_omega_theme_libraries_info() {
  $libraries['modernizr'] = array(
    'name' => t('Modernizr'),
    'description' => t('Modernizr is a JavaScript library that detects HTML5 and CSS3 features in the user?s browser.'),
    'vendor' => 'Modernizr',
    'vendor url' => 'http://modernizr.com/',
    'package' => t('Polyfills'),
    'files' => array(
      'js' => array(
        'modernizr.min.js' => array(
          'weight' => 130,
          'every_page' => TRUE,
        ),
      ),
    ),
    'variants' => array(
      'source' => array(
        'name' => t('Source'),
        'description' => t('During development it might be useful to include the source files instead of the minified version.'),
        'files' => array(
          'js' => array(
            'modernizr.js' => array(
              'weight' => 130,
              'every_page' => TRUE,
            ),
          ),
        ),
      ),
    ),
  );

  return $libraries;
}

/**
 * Create variables for custom panels panes created via FPP
 **
 * Implements hook_preprocess_fieldable_panels_pane().
 */

function wcm_omega_preprocess_fieldable_panels_pane(&$vars) {
  $type = $vars['elements']['#bundle'];

  // Tile panes
  if (in_array($type, array('tile_pane', 'tile_pane_plus_text_area'))) {
    if (!empty($vars['content']['title'])) {
      $vars['tile_title'] = $vars['content']['title']['#value'];
    }
    $vars['tile_url'] = NULL;
    if (!empty($vars['content']['field_tile_link'])) {
      $link = $vars['content']['field_tile_link']['#items'][0];
      $vars['tile_url'] = url($link['url'], array('absolute' => TRUE, 'fragment' => $link['fragment'], 'query' => $link['query']));
    }
    if (!empty($vars['content']['field_background_color'])) {
      $vars['classes_array'][] = $vars['content']['field_background_color']['#items'][0]['value'];
    }
    if (!empty($vars['content']['field_background_color_2'])) {
      $vars['classes_array'][] = $vars['content']['field_background_color_2']['#items'][0]['value'];
    }
    if (!empty($vars['content']['field_tile_image_style'])) {
      $style = $vars['content']['field_tile_image_style']['#items'][0]['value'];
      $vars['tile_icon'] = ($style == 'icon');

      if (!empty($style)) {
        $vars['classes_array'][] = 'tile-image-style-' . $style;
      }
    }
    if (!empty($vars['tile_url'])) {
      $vars['classes_array'][] = 'tile-pane-linked';
    }
  }

  // Image panes
  elseif ($type == 'image') {
    if (!empty($vars['content']['field_basic_image_image'])) {
      $vars['content']['field_basic_image_image'][0]['#path']['path'] = !empty($vars['elements']['#fieldable_panels_pane']->field_ocio_link) ? $vars['elements']['#fieldable_panels_pane']->field_ocio_link['und'][0]['display_url'] : NULL;
    }
  }
}

/**
 * Add classes to field_banner_image_text
 **
 * Implements hook_preprocess_field().
 */

function wcm_omega_preprocess_field(&$vars) {

  if ($vars['element']['#field_name'] == 'field_banner_image_text') {

    //print classes based on selected options from field
    if (isset($vars['element']['#object']->field_banner_image_text_color['und'][0]['value'])) {
      $vars['classes_array'][] = $vars['element']['#object']->field_banner_image_text_color['und'][0]['value'];
    }
    if (isset($vars['element']['#object']->field_banner_image_text_location['und'][0]['value'])) {
      $vars['classes_array'][] = $vars['element']['#object']->field_banner_image_text_location['und'][0]['value'];
    }
    if (isset($vars['element']['#object']->field_banner_image_text_bkground['und'][0]['value'])) {
      $vars['classes_array'][] = $vars['element']['#object']->field_banner_image_text_bkground['und'][0]['value'];
    }
  }
}

/**
 * Implements template_preprocess_panels_pane().
 */
function wcm_omega_preprocess_panels_pane(&$vars) {
  $pane = $vars['pane'];

  if ($pane->type == 'mm_widgets_live_pane') {
    $vars['channel'] = '';
    if (!empty($pane->configuration['single_channel'])) {
      $vars['channel'] = $pane->channel;
      $vars['classes_array'][] = 'mm-single-channel';
      if ($pane->channel == 'fw' ) {
        $vars['classes_array'][] = 'mm-channel-type-undefined';
      }
    }
  }
}
